@@ -1,9 +1,15 @@
 
 import platform
 import sys
-import sysconfig
 import os
 
+# The 'sysconfig' module is only available with Python 2.7 and newer, but
+# an equivalent module in 'distutils' is available for Python 2.6.
+if sys.version < '2.7':
+  from distutils import sysconfig
+else:
+  import sysconfig
+
 # The 'imp' module is deprecated since Python 3.4, and the use of
 # 'importlib' is recommended instead.
 if sys.version < '3.4':
@@ -33,9 +39,9 @@ config = {
   "Prefix"           : getattr(sys, "prefix", ""),
   "ExecPrefix"       : getattr(sys, "exec_prefix", ""),
   "BaseExecPrefix"   : getattr(sys, "base_exec_prefix", ""),
-  "PythonPath"       : pathsep.join(sys.path[1:]),
+  "PythonPath"       : pathsep.join((x or "." for x in sys.path)),
   "LIBPL"            : sysconfig.get_config_var("LIBPL"),
-  "LIBDIR"           : sysconfig.get_config_var("LIBDIR"),
+  "LIBDIR"           : sysconfig.get_config_var("LIBDIR")
 }
 
 # Read numpy configuration (if available)
